/*
 * 쐬: 2005/06/04
 *
 */
package org.seasar.buri.engine;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.seasar.buri.oouo.internal.structure.BuriActivityType;
import org.seasar.buri.oouo.internal.structure.BuriPackageType;
import org.seasar.buri.oouo.internal.structure.BuriWorkflowProcessType;
import org.seasar.framework.util.StringUtil;

/**
 * @author makotan
 *
 */
public class BuriRealPath {
    private String workflowPackage = "";
    private String workflowProcess = "";
    private ArrayList activity = new ArrayList();
    private String pathStr = "";

    public BuriRealPath() {
        this("");
    }

    public BuriRealPath(String path) {
        if(StringUtil.isEmpty(path)) {return;}
        StringTokenizer st = new StringTokenizer(path, ".");
        setPathFromToken(st);
        pathStr = path;
    }
    
    public BuriRealPath copyRealPath() {
        BuriRealPath clonePath = new BuriRealPath();
        clonePath.workflowPackage = workflowPackage;
        clonePath.workflowProcess = workflowProcess;
        clonePath.activity.addAll(activity);
        clonePath.pathStr = pathStr;
        return clonePath;
    }
    

    protected void setPathFromToken(StringTokenizer st) {
        if(st.hasMoreTokens()) {
            workflowPackage = st.nextToken();
        }
        if(st.hasMoreTokens()) {
            workflowProcess = st.nextToken();
        }
        ArrayList activity = new ArrayList();
        while(st.hasMoreTokens()) {
            activity.add(st.nextToken());
        }
        this.activity = activity;
        setupPlainName();
    }
    public void changePash(String newPath,int pos) {
        if(StringUtil.isEmpty(newPath)) { 
            newPath=""; 
        }
        if(activity.size()==0) {
            getActivity().add(newPath);
        } else {
            assert pos < activity.size();
            activity.set(pos,newPath);
        }
        setupPlainName();
    }
    public void changePath(BuriActivityType newAct,int pos) {
        changePash(newAct.getName(),pos);
    }
    
    public void moveChildPath(String childPath) {
        activity.add(childPath);
        setupPlainName();
    }
    public void moveChildPath(BuriActivityType newAct) {
        moveChildPath(newAct.getName());
        setupPlainName();
    }
    
    public void moveUpPath() {
        if(activity.size()==0) {
            return;
        } else {
            activity.remove(activity.size()-1);
        }
        setupPlainName();
    }
    
    public List getActivity() {
        return new ArrayList(activity);
    }
    public void setActivity(List activity) {
        activity.clear();
        activity.addAll(activity);
        setupPlainName();
    }
    public String getWorkflowProcess() {
        return workflowProcess;
    }
    public void setWorkflowProcess(String process) {
        if(StringUtil.isEmpty(process)) { 
            workflowProcess ="";
        } else {
            workflowProcess = process;
        }
        setupPlainName();
    }
    public void setWorkflowProcess(BuriWorkflowProcessType wkfProcess) {
        setWorkflowProcess(wkfProcess.getName());
        setupPlainName();
    }
    public String getWorkflowPackage() {
        return workflowPackage;
    }
    public void setWorkflowPackage(String workflow) {
        if(StringUtil.isEmpty(workflow)) {
            workflowPackage ="";
        } else {
            workflowPackage = workflow;
        }
        setupPlainName();
    }
    public void setWorkflowPackage(BuriPackageType packageDoc) {
        setWorkflowPackage(packageDoc.getName());
        setupPlainName();
    }
    
    public String getPlainName() {
        return pathStr;
    }
    
    protected void setupPlainName() {
        String path = workflowPackage + "." + workflowProcess;
        String act = "";
        String koron = "";
        Iterator ite = activity.iterator();
        while(ite.hasNext()) {
            act = act + koron + ite.next();
            koron = ".";
        }
        pathStr = path + koron + act;
    }
    
    public String toString() {
        return pathStr;
    }
    
    protected boolean isCorrect() {
        if(hasPathName(getWorkflowPackage())) {
            if(hasPathName(getWorkflowProcess())) {
                return isCorrectActivitys(getActivity());
            }
        }
        return false;
    }
    
    protected boolean isCorrectActivitys(List activitis) {
        Iterator ite = activitis.iterator();
        while(ite.hasNext()) {
            String pathName = (String)ite.next();
            if( ! hasPathName(pathName)) {
                return false;
            }
        }
        return true;
    }
    
    protected boolean hasPathName(String pathName) {
        return pathName.length() > 0;
    }

    /**
     * Override hashCode.
     *
     * @return the Objects hashcode.
     */
    public int hashCode() {
        int hashCode = 1;
        hashCode = 31
            * hashCode
            + (workflowPackage == null ? 0 : workflowPackage.hashCode());
        hashCode = 31
            * hashCode
            + (workflowProcess == null ? 0 : workflowProcess.hashCode());
        hashCode = 31 * hashCode + (activity == null ? 0 : activity.hashCode());
        hashCode = 31 * hashCode + (pathStr == null ? 0 : pathStr.hashCode());
        return hashCode;
    }
    
    
    
}
