/*
 * 쐬: 2006/03/26
 *
 */
package org.seasar.coffee.script.ognl.impl;

import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Map;

import ognl.Ognl;
import ognl.OgnlException;

import org.seasar.buri.exception.BuriException;
import org.seasar.coffee.script.Script;
import org.seasar.coffee.script.ScriptFactory;
import org.seasar.coffee.script.exception.ScriptCompleException;
import org.seasar.coffee.script.exception.ScriptExecuteException;
import org.seasar.framework.container.S2Container;
import org.seasar.framework.log.Logger;
import org.seasar.framework.util.InputStreamReaderUtil;
import org.seasar.framework.util.ReaderUtil;
import org.seasar.framework.util.StringUtil;

public class OgnlScriptImpl implements Script {
    private static Logger logger = Logger.getLogger(OgnlScriptImpl.class);
    private S2Container container;
    private ScriptFactory factory;
    
    private Map getContext(Map context) {
        if(context!=null) {
            return context;
        }
        return factory.getContext();
    }
    
    public Object eval(Object tgtObj,String expression, Map context) {
        context = getContext(context);
        Object compile = compileExpression(expression);
        Object result = eval(tgtObj,compile, context);
        return result;
    }

    public Object eval(Object tgtObj,Object compileResult, Map context) {
        context = getContext(context);
        Object ret=null;
        if(tgtObj==null) {
            tgtObj = container.getRoot();
        }
        try {
            Map ognlContext = Ognl.createDefaultContext(null);
            ognlContext.putAll(context);
            ret = Ognl.getValue(compileResult, ognlContext, tgtObj);
        } catch (OgnlException t) {
            throw new ScriptExecuteException("ESCS0002",t.getLocalizedMessage(),context,t);
        } catch (Exception re) {
            if(re instanceof BuriException) {
                throw (BuriException)re;
            }
            throw new ScriptExecuteException("ESCS0002",re);
        }
        return ret;
    }

    public void setVal(Object tgtObj, String script, Object setObj, Map context) {
        context = getContext(context);
        Object compile = compileExpression(script);
        setVal(tgtObj, compile, setObj,context);
    }

    public void setVal(Object tgtObj, Object compileResult, Object setObj,
            Map context) {
        context = getContext(context);
        if(tgtObj==null) {
            tgtObj = container.getRoot();
        }
        try {
            Map ognlContext = Ognl.createDefaultContext(null);
            ognlContext.putAll(context);
            Ognl.setValue(compileResult, ognlContext, tgtObj,setObj);
            context.putAll(ognlContext);
    
        } catch (OgnlException t) {
            throw new ScriptExecuteException("ESCS0003",t.getLocalizedMessage(),context,t);
        } catch (RuntimeException re) {
            if(re instanceof BuriException) {
                throw re;
            }
            throw new ScriptExecuteException("ESCS0003",re.getLocalizedMessage(),context,re);
        }
    }

    public Object run(InputStream in, Object tgtObj, Map context) {
        context = getContext(context);
        InputStreamReader isR = InputStreamReaderUtil.create(in);
        String script = ReaderUtil.readText(isR);
        Object result = run(script, tgtObj,context);
        return result;
    }

    public Object run(String script, Object tgtObj, Map context) {
        context = getContext(context);
        return processOgnls(script,tgtObj,context);
    }

    public Object run(Object compileResult, Object tgtObj, Map context) {
        context = getContext(context);
        return run(compileResult.toString(), tgtObj,context);
    }

    public Object compileExpression(String expression) {
        Object ret=null;
        try {
            ret = Ognl.parseExpression(expression);
        } catch (OgnlException e) {
            throw new ScriptCompleException(expression,"ESCS0001","",e);
        }
        return ret;
    }

    public Object compileSetExpression(String setexpression) {
        return compileExpression(setexpression);
    }

    public Object compileScript(String script) {
        return script;
    }

    
    
    
    private Object processOgnls(String ognls,Object tgtObj, Map contextData) {
        if(StringUtil.isEmpty(ognls)) {
            return null;
        }
        String ognlLines[] = ognls.split("\n");
        Object result = null;
        for(int i=0; i < ognlLines.length ; i++) {
            String ognlLine = ognlLines[i];
            if( isProcessedOgnl(ognlLine) ) { //ognlLine.length()>0) {
                result = processOgnl(ognlLine,i,tgtObj,contextData);
            }
        }
        return result;
    }
    
    private boolean isProcessedOgnl(String ognlLine) {
        String line = ognlLine.trim();
        if(line.trim().length()==0) {
            return false;
        }
        String commentCheck = "";
        if(line.length() >= 2) {
            commentCheck = line.substring(0,2);
        }
        if(commentCheck.compareTo("//") == 0) {
            if(logger.isDebugEnabled()) {
                logger.debug(ognlLine);
            }
            return false;
        }
        return true;
    }
    
    private Object processOgnl(String ognlLine,int line,Object tgtObj, Map contextData) {
        String ognls[] = ognlLine.split("=");
        String setContext = null;
        String getValue = "";
        if(ognls.length==1) {
            getValue = ognlLine;
        }else if(ognls.length==2) {
            setContext = ognls[0].trim();
            getValue = ognls[1].trim();
        }
        Object data = getValue(getValue,line,tgtObj,contextData);
        setValue(setContext,data,contextData);
        return data;
    }
    private Object getValue(String getValue,int line,Object tgtObj, Map contextData) {
        String getValStr = "";
        if(logger.isDebugEnabled()) {
            getValStr = "Line:" + line + "//=["+getValue+"]";
        }
        Object data = null;
        try {
            data = eval(tgtObj,getValue,contextData);
        }finally {
            if(logger.isDebugEnabled()) {
                logger.debug(getValStr + " :" + data);
            }
        }
        return data;
    }
    
    private void setValue(String setContext,Object data,Map contextData) {
        if(setContext != null) {
            if(logger.isDebugEnabled()) {
                logger.debug(setContext + " = " + data);
            }
            setVal(null,setContext,data,contextData);
        }
    }

    public S2Container getContainer() {
        return container;
    }

    public void setContainer(S2Container container) {
        this.container = container;
    }

    public ScriptFactory getFactory() {
        return factory;
    }

    public void setFactory(ScriptFactory factory) {
        this.factory = factory;
    }

}
